/* This file was generated by syncqt. */
#ifndef QT_QTPRINTSUPPORT_VERSION_H
#define QT_QTPRINTSUPPORT_VERSION_H

#define QTPRINTSUPPORT_VERSION_STR "5.15.13"

#define QTPRINTSUPPORT_VERSION 0x050F0D

#endif // QT_QTPRINTSUPPORT_VERSION_H
